#!/usr/bin/env python3
"""
Template Me - module with template for your favourite projects.
"""


def get_version():
    """ Get version of protemp package. """
    return "0.0.7"

__version__ = get_version()
